import { PropsWithChildren } from 'react'
import Taro, { useDidHide, useDidShow, useLaunch } from '@tarojs/taro'
import { globalApp } from '@/class/GlobalContext'
import {
    isEasyMode, createSessionId, formatQuery, formatDate, getOption
} from '@/utils'
import useHotLaunch from '@/hooks/useHotLaunch'
import http from '@/http/index'
import {
    getWxUserInfoByCode, getAbtestConfig,
    generateMidv2, getUserFrontConfig
} from '@/http/api'
import { userActivie } from '@/logger/userActivieLog'
import { eventId } from '@/logger/const'
import { Provider } from 'react-redux'
import store from './store'


import './app.less'

function App({ children }: PropsWithChildren<any>) {

    useLaunch(async (launch) => {
        globalApp()
        console.log('hhz - launch option:', launch)

        // create session id
        updateSession()

        updateGlobal(launch)

        // // network change
        // listeningConnectStatus()

        // userinfo
        await updateUserInfo()

        // mid
        const mid = await requestMid()

        Taro.$global.set('mid', mid)
        Taro.setStorageSync('mid', mid)

        //
        getAccountInfoInApp()

        // system info
        updateSystem()
    })


    useDidShow((options) => {
        updateAbGroup()
        reportUserActiveLog(options)
        // onEnvByShake()
    })

    useDidHide(() => {
        // offEnvByShake()

        Taro.$abGroupInstance
            .cleanLoop()
            .cleanStack()


        const id = createSessionId()
        Taro.$global.set('subSessionId', id)
    })

    useHotLaunch((options) => {
        Taro.$global.set('subSessionid', createSessionId())

        const launchOption = Taro.$global.get('launchOption')
        Taro.$global.set('launchOption', {
            ...launchOption,
            hotScene: options.scene
        })

        // reportShareVideo()
    })
    // children 是将要会渲染的页面
    return (
        <Provider store={store} >
            {children}
        </Provider>
    )
}

function updateGlobal(launch) {
    Taro.$global.set('launchOption', {
        ...launch,
        hotScene: launch.scene
    })

    // 三十天有无回流
    Taro.$global.set('isReturningUser', !!Taro.getStorageSync('$RETURNING'))
}

// 启动埋点上报
function reportUserActiveLog(options) {
    const launchOption = Taro.$global.get('launchOption')

    options = getOption(options)

    const { scene = -1, hotScene } = launchOption

    if (scene === 1154)
        options.pageSource = options.pageSource || options.path + '-wxTimeline'

    userActivie(eventId.userActiveAction, {
        option: JSON.stringify(options),
        senceType: scene,
        hotLaunchSceneType: hotScene,
        activeDate: formatDate(new Date(), 'yyyy-MM-dd'),
        activeTimestamp: new Date().getTime(),
        businessType: 'path'
    })
}

function updateAbGroup() {
    Taro.$abGroupInstance
        .listen(fetchAbtestConfig)
        .loop()
        .emit()
}
// ab 实验信息
function fetchAbtestConfig() {
    http.post(getAbtestConfig).then((res: RequestType) => {
        const { data, code } = res
        if (code !== 0)
            return

        Taro.$abGroupInstance.storeGroup(data)
        // Taro.setStorageSync('$EXPERIMENT_CONFIG', data)
    })
}
// system
function updateSystem() {
    const systemInfo = Taro.getSystemInfoSync()

    Taro.$global.set('systemInfo', {
        ...systemInfo,
        easyMode: isEasyMode(systemInfo) && 2 || 1// 关怀模式
    })
}
// userinfo
async function updateUserInfo() {
    const userInfo = Taro.getStorageSync('$USER_INFO')

    if (userInfo) {
        Taro.$global.set('userInfo', userInfo)
        return Promise.resolve(userInfo)
    }

    return Taro.loginSync()
        .then((res: Taro.login.SuccessCallbackResult) => {
            const { code } = res
            return http.post(getWxUserInfoByCode, { code })
        }).then((res: RequestType) => {
            const { code, data } = res

            if (code !== 0)
                return

            Taro.$global.set('userInfo', data)
            Taro.setStorageSync('$USER_INFO', data)

            return data
        }).catch((err) => {
            console.log(err)
        })
}

// sessionid 更新
function updateSession() {
    const id = createSessionId()
    Taro.$global.set('sessionId', id)
    Taro.$global.set('subSessionId', id)
}

async function requestMid() {
    const mid = Taro.getStorageSync('mid')

    if (mid)
        return mid

    return Taro.loginSync()
        .then((res: Taro.login.SuccessCallbackResult) => {
            const { code } = res
            return http.post(generateMidv2, { code })
        }).then((res: RequestType) => {
            const { code, data } = res

            if (code !== 0)
                return

            return data
        }).catch(err => {
            console.log(err)
        })
}

// 30天无回流
function getAccountInfoInApp() {
    http.post(getUserFrontConfig).then((res: RequestType) => {
        const { code, data } = res
        if (code !== 0)
            return

        const { has30DayReturnCnt } = data
        // 用户在小程序内的各种属性状态
        Taro.$global.set('accountInApp', data)

        // 30天有无回流
        Taro.setStorageSync('$RETURNING', has30DayReturnCnt === 1)
        Taro.$global.set('isReturningUser', has30DayReturnCnt === 1)

    }).catch(err => {
    })
}

export default App
