import Taro from '@tarojs/taro'
import { json } from 'stream/consumers'

export const WX_AB_GROUP = { ab_test010: 'ab100' }
for (let i = 1; i < 10; i++)
    WX_AB_GROUP[`ab_test00${i}`] = 'ab100'

class AbGroupContext implements AbGroupContextType {
    #group: object = WX_AB_GROUP
    #evtStack: Array<Function> = []
    #timer: NodeJS.Timeout | null
    #groupMap = new Map()

    constructor() {
        this.#group = {
            ...this.#group,
            ...Taro.getExptInfoSync()
        }
        this.#groupMap = this.objToMap(Taro.getStorageSync('$EXPERIMENT_CONFIG'))
    }

    listen(evt) {
        if (typeof evt !== 'function')
            return this

        this.#evtStack.push(evt)

        return this
    }

    emit() {
        this.#evtStack.forEach(evt => evt())

        return this
    }

    loop(time: number = 60000) {
        if (this.#timer)
            return this

        this.#timer = setInterval(() => {
            this.emit()
        }, time)

        return this

    }

    cleanLoop() {
        if (this.#timer)
            clearInterval(this.#timer)

        return this
    }

    cleanStack() {
        this.#evtStack = []

        return this
    }

    get() {
        return this.#group
    }

    storeGroup(config) {
        let abInfos = {}
        Object.keys(config).forEach(level => {
            let group = this.#group[level]
            const abGroupConfigArray = config[level][group] || []
            abGroupConfigArray.map(item => {
                try {
                    item.configValue = JSON.parse(item.configValue)
                } catch (error) {
                    item.configValue = {}
                }
                return item
            })
            abInfos[level] = abGroupConfigArray
        })

        this.#groupMap = this.objToMap(abInfos)
        Taro.setStorageSync('$EXPERIMENT_CONFIG', abInfos)
    }

    objToMap(obj) {
        let map = new Map()
        Object.keys(obj).forEach(level => {
            let abExpInfos = obj[level]
            abExpInfos.forEach(abExpInfo => {
                map.set(abExpInfo.abExpCode, {
                    level: level,
                    group: this.#group[level],
                    abExpCode: abExpInfo.abExpCode,
                    config: abExpInfo.configValue
                })

            });

        })
        return map

    }

    validExperiment(name) {
        return this.#groupMap.has(name + '')
    }

    getExperimentConfigByName(name) {
        return this.#groupMap.get(name + '')
    }

}

export default AbGroupContext
