import Taro, { useLoad, useDidShow, useReady, useDidHide, usePullDownRefresh, useShareAppMessage, useRouter } from '@tarojs/taro'
import { useMemo, useState } from 'react'
import type CustomTabBar from '@/custom-tab-bar'
import { View } from '@tarojs/components'
import { VideoInfo } from '@/constants/commentTypes'
import { videoListV2 } from '@/http/api'
import VideoSwiper from '@/components/VideoSwiper'
import { sharePageAppMessage } from '@/shareHelper'
import Route from '@/class/Route'
import { filterCardProps, pxToRpx } from '@/utils/index'
import { CATEGORY_PAGESOURCE } from '@/const/index'

// import { getTopSafeHeight, throttle, formatSecondsAsTime, getPreIds } from '@/utils'

// import Route from '@/class/Route'
import './index.less'

let pageNo = 1
let activeIndex = 0

export default function Index() {
    const page = useMemo(() => Taro.getCurrentInstance().page, []);
    const router = useRouter()
    const { params } = router || {}

    const [videoList, setVideoList] = useState<VideoInfo[]>([])
    const [refresherTriggered, setRefresherTriggered] = useState(false)
    const [needResumePlay, setNeedResumePlay] = useState(false)
    const [toSharePage, setToSharePage] = useState(false)
    const [flushList, setFlushList] = useState(false)
    const [botHeight, setBotHeight] = useState(0)
    useLoad(() => {
        const { redirect } = params
        let dur = 0
        if (redirect) {
            const route = new Route()
            route.push(decodeURIComponent(redirect))
            dur = 1000
            setToSharePage(true)
        }

        setTimeout(() => {
            fecthVideoListV2()
        }, dur);

        const systemInfo = Taro.$global.get('systemInfo')
        let { screenHeight = 0 } = systemInfo
        let { bottom = 0 } = systemInfo.safeArea || {}
        setBotHeight(screenHeight - bottom)
    })

    useDidShow(() => {
        const tabbar = Taro.getTabBar<CustomTabBar>(page)
        tabbar?.setSelected(0)

        if (videoList.length > 0 && toSharePage) {
            setToSharePage(false)
            setNeedResumePlay(!needResumePlay)
        }
    })

    useReady(() => {
    })

    useDidHide(() => {

    })

    usePullDownRefresh(() => {
        console.log('usePullDownRefresh')
    })

    useShareAppMessage((shareRes) => {
        return sharePageAppMessage({
            video: videoList[activeIndex],
            activeIndex,
            router,
            shareRes
        })
    })

    // const fecthVideoList = throttle(fecthVideoListV2)
    function fecthVideoListV2(cleanOldCardList = false) {
        if (cleanOldCardList)
            pageNo = 1

        Taro.$http.post(videoListV2, {
            categoryId: 55,
            pageNo,
            pageSize: 6,
            pageSource: 'pages/category'
        }).then((res: RequestType) => {
            const { code, data } = res
            if (code != 0)
                return

            let oldCardList = videoList

            if (cleanOldCardList) {
                oldCardList = []
                setRefresherTriggered(false)
            }

            pageNo++

            setVideoList([
                ...oldCardList,
                ...filterCardProps(data, CATEGORY_PAGESOURCE)
            ])
            if (cleanOldCardList) {
                setFlushList(true)
                setTimeout(() => {
                    setFlushList(false)
                }, 300);
            }

        }).catch(() => {
        })
    }


    function onAnimationFinish(current) {
        if (current == -99) {
            fecthVideoListV2(true)

            return
        }
        activeIndex = current
        if (current >= videoList.length - 4) {
            fecthVideoListV2()
        }
    }

    return (
        <View className='category-page' style={{ height: 'calc(100% - ' +  `${Math.floor(pxToRpx(55))}px - ` + `${botHeight}px);` }}>
            <VideoSwiper
                list={videoList}
                flushList={flushList}
                needResumePlay={needResumePlay}
                onFinish={onAnimationFinish}
            />

        </View>
    )
}
