import Taro from '@tarojs/taro'
import { stringify, S4 } from '@/utils'
import useHotLaunch from '@/hooks/useHotLaunch'
import { RECOMMEND_PAGESOURCE, USER_SHARE_PAGESOURCE, CATEGORY_PAGESOURCE, DETAIL_PAGESOURCE, DETAIL_RECOMMEND } from '@/const'
import { shareReport, weixinFriend, shareClickReport } from '@/logger'
import { USER_SENCE } from '@/const/index'

export function sharePageAppMessage({ video, activeIndex, router, shareRes }) {
    useHotLaunch.valid = false
    const { path, params } = router
    const launchOption = Taro.$global.get('launchOption')
    const { target } = shareRes || {}
    const { dataset } = target || {}

    return shareVideoToWechat({
        path,
        video,
        pageSource: USER_SHARE_PAGESOURCE,
        recommendPageType: RECOMMEND_PAGESOURCE,
        parentRootPageSource: CATEGORY_PAGESOURCE,

        rootPageSource: params.rootPageSource,
        shareDepth: params.shareDepth,
        rootPageTimestamp: params.rootPageTimestamp,
        rootShareId: params.rootShareId,
        rootMid: params.rootMid,
        rootLaunchShareId: params.rootLaunchShareId,
        parentEventIds: params.eventIds,

        actionPosition: activeIndex,
        scene: launchOption.scene,
        shareButtonType: dataset?.buttonType || 0,
        prePageSource: '',
        userSence: params.userSence
    })
}

export function shareVideoToWechat(params) {
    const { video } = params
    const mid = Taro.$global.get('mid')
    const shareId = mid + '-' + S4() + new Date().getTime()

    // TODO: 处理 pagesource
    let pageSource = video.head ? USER_SHARE_PAGESOURCE : RECOMMEND_PAGESOURCE

    if (params.userSence !== USER_SENCE.share)
        pageSource = video.head ? DETAIL_PAGESOURCE : DETAIL_RECOMMEND

    const userInfo = Taro.$global.get('userInfo')

    const query = {
        userSence: USER_SENCE.share,
        mid: mid,
        rootMid: params.rootMid || mid,
        shareId,
        pageSource,
        rootPageSource: params.rootPageSource || pageSource,
        flowPool: video.flowPool || '',
        launchscene: params.scene,
        su: userInfo.uid,
        recomTraceId: video.recomTraceId || '',
        rootPageTimestamp: params.rootPageTimestamp || new Date().getTime(),
        rootShareId: params.rootShareId || shareId,

        shareDepth: +params.shareDepth + 1 || 0,
        isRecommendShare: video.isRecommendShare,
        recommendSource: video.recommendSource,
        recommendLogVO: video.recommendLogVO || '{}',
        shareImageId: video.shareImgId,
        shareTitleId: video.titleId,
        shareButtonType: params.shareButtonType, // 按钮类型
        ...videoPlayParams(video)
    }

    const detailPath = `/pages/share/share?${stringify(query)}`

    shareReport(query)
    weixinFriend(query)

    return {
        title: video.title || '好友分享给你一个视频，点击查看～',
        imageUrl: video.shareImgPath,
        path: `/pages/category/index?redirect=${encodeURIComponent(detailPath)}`
    }
}

function videoPlayParams(video) {
    const {
        id, title, shareImgPath, videoPath,
    } = video

    return {
        title: encodeURIComponent(title || ''),
        videoPath: encodeURIComponent(videoPath),
        id,
        shareImgPath: encodeURIComponent(shareImgPath)
    }
}

// 分享回流上报
export function returnedCrowdClickReport(params) {
    if (params.userSence !== USER_SENCE.share)
        return

    shareClickReport({
        shareId: params.shareId,
        clickObjectId: params.id || 0,
        pageSource: params.pageSource,
        rootPageSource: params.rootPageSource,
        shareButtonType: params.shareButtonType,
        shareDepth: params.shareDepth || 1,
        rootLaunchShareId: params.rootLaunchShareId || '',
        rootShareId: params.rootShareId || '',
        shareTitle: params.title,
        shareImageUrl: params.shareImgPath,
        shareTitleId: params.shareTitleId,
        shareImageId: params.shareImageId,
        parentShareId: params.parentShareId,
        extJson: JSON.stringify({
            parentRootPageSource: params.parentRootPageSource || params.rootPageSource,
        })
    })
}
