import { useState } from 'react'
import Taro, { useLoad, useRouter } from '@tarojs/taro'
import { View, Textarea, Button } from "@tarojs/components"
import { reasonList, videoReport } from '@/http/api/index'
import './index.less'

function Complaining() {
    let reason = ''
    const { params } = useRouter()
    const { videoId } = params
    const [list, setList] = useState([])
    const [showOtherReason, setShowOtherReason] = useState(false)
    useLoad(() => {
        fetchList()
    })

    function fetchList() {
        Taro.$http.post(reasonList).then((res: RequestType) => {
            const { code, data } = res
            if (code !== 0)
                return

            const reasons = data.reduce((calc, cur) => {
                return calc.concat(cur.reasons)
            }, [])

            setList(reasons)
        })
    }

    function onClick(item, index) {
        if (list.length - 1 === index) {
            setShowOtherReason(true)
            return
        }

        showOtherReason && setShowOtherReason(false)

        reason = item

        onSubmit()
    }

    function textInput(res) {
        const { detail } = res

        reason = detail?.value
    }

    function onSubmit() {
        if (!reason) {
            Taro.showToast({
                title: '请填写举报理由',
                icon: 'none'
            })
            return
        }

        Taro.$http.post(videoReport, { videoId }).then((res: RequestType) => {
            const { code, msg } = res

            if (code !== 0) {
                Taro.showToast({
                    title: msg,
                    icon: 'none'
                })
                return
            }

            Taro.showToast({
                title: '举报成功',
                icon: 'none'
            })

            setTimeout(() => {
                Taro.navigateBack()
            }, 500)
        })
    }

    return (
        <View className='complaining-page'>
            <View className='complaining-comment'>
                此投诉为本小程序自有投诉渠道，非微信官方投诉渠道
            </View>

            {
                list.map((item, index) => {
                    return (
                        <View className='reason' onClick={() => onClick(item, index)}>
                            {item}
                        </View>
                    )
                })
            }

            {
                showOtherReason && <View className='other-reason'>
                    <Textarea autoFocus placeholderClass='placeholder-class' placeholder='填写举报理由' onInput={textInput} />

                    <Button className='submit' onClick={onSubmit}>提交</Button>
                </View>
            }
        </View>
    )
}

export default Complaining
